#ifndef GuassianLpSolver_h_included_p
#define GuassianLpSolver_h_included_p

#include <memory>  // to get size_t
#include <stdlib.h>
#include <portable_math.h>  // math.h and float.h and other stuff to make it portable

namespace cxxtls
{

class GaussianLpSolver
   /// This class is a wrapper around the data structures and algorithms needed to solve
   /// a set of simultaneous linear equations using Gauss' method.  It was derived from
   /// some fortran code I found in "Numerical Computing:  an introduction", by Shampine and
   /// Allen.  Copyright 1973 W. B. Saunders.  Page 254.  Program "lassol()".
   ///
   /// To use this class, construct it, giving it the number equations and unknowns.
   ///
   /// Then call the solve() function.  If it returns true, you have a solution which
   /// is found in array variables_.
   ///
   /// The equations are defined as a vector of n+1 doubles per row times N
   /// rows.  The data has the format:
   ///
   ///@code
   ///    aX + bY +cZ = A    // equation 1
   ///    dX + eY +eZ = B    // equation 2
   ///    fX + gY +hZ = C    // equation 3
   ///@endcode
   ///
   /// To populate the tables in the solver, use the following functions:
   ///
   /// @setParameter()
   /// @setRHS()
   ///
   /// The parameters populate the square grid of parameters.  In the above example,
   /// you would make the following calls:
   ///
   ///@code
   ///
   ///    GaussianLpSolver g(3);
   ///
   ///    g.setParameter(0, 0, a);   // equation 1
   ///    g.setParameter(1, 0, b);
   ///    g.setParameter(2, 0, c);
   ///    g.setRHS(0, A);            // sets the right hand side of the above equation.
   ///    
   ///    g.setParameter(0, 1, d);   // equation 2
   ///    g.setParameter(1, 1, e);
   ///    g.setParameter(2, 1, f);
   ///    g.setRHS(1, B);
   ///    
   ///    g.setParameter(0, 1, f);   // equation 3
   ///    g.setParameter(1, 1, g);
   ///    g.setParameter(2, 1, h);
   ///    g.setRHS(2, C);
   ///
   ///@endcode
   ///
   ///    You can then call "solve()", and extract the results like this:
   ///
   ///@code
   ///
   ///    if(!g.solve())
   ///       throw error("....");
   ///
   ///    cout << "X = " << g.result(0) << endl;
   ///    cout << "Y = " << g.result(1) << endl;
   ///    cout << "Z = " << g.result(2) << endl;
   ///
   ///@endcode
   ///
   /// 
{
    bool     solved_;    ///<  true if the solve() function has returned true.
    int      n_;         ///<  the number of equations and of unknowns
    double **matrix_;    ///<  each row defines the parameters of one of the equations
    double *rhs_;        ///<  the other part of each eqation.  This column vector defines the values the matrix rows equate to.
    double *variables_;  ///<  the resultant set of variable values which are computed by the solve() function.

public:

    ~GaussianLpSolver()
    {
       for(int i = 0; i < n_; ++i)
          delete matrix_[i];

       delete [] matrix_;
       delete [] rhs_;
       delete [] variables_;

    }


    GaussianLpSolver(int n)
        /// Construct a solver object and specify the number of equations and unknowns.
        ///
        /// You must populate the parameters and the right hand sides of n equations by
        /// making calls to setParameter(parmNumber, rowNumber, value) and setRHS(rowNumber, value.)
        ///
        /// When populated, you call the solve function.  If it returns true, you can then call
        /// result(parameterId) to get  the values of the variables calculated.
    :   solved_(false)
    ,   n_(n)
    {
        // create the matrices of interest needed to solve an NxN 
        // equation set.

        variables_ = new double[n_];
        rhs_ = new double[n_];
        matrix_ = new double*[n_];

        for(int i=0; i < n_; i++)
        {
            matrix_[i] = new double[n_];

        }

        for(int i=0; i<n_; i++)
        {
            for(int j=0; j<n_; j++)
            {
                matrix_[i][j]=0;
            }
            rhs_[i]=0;
            variables_[i]=0;
        }
        
    }

    template<class Stream>
    friend Stream &operator<< (Stream &s, GaussianLpSolver const &me)
    {
       for(int i = 0; i < me.n_; ++i)
       {
           for(int j = 0; j < me.n_; ++j)
               s << me.matrix_[i][j] << ' ';

           s << me.rhs_[i] << '\n';
       }


       if(me.solved_)
       {
           s << "Solution:\n    ";
           
           for(int i = 0; i < me.n_; ++i)
              s << me.variables_[i]  << ' ';
           
           s << '\n';
       }
       else
       {
           s << "not yet solved.\n";
       }

       return s;
    }


   bool solve()
      /// solve and return true on success.
   {
      evaluate_diag();

      if(!evaluate_rows())
         return false;

      evaluate_solution();

      for(int i = 0; i < n_; ++i)
      {
          if(isnan(variables_[i]))
             return false; // can't solve the equation.
          
      }

      solved_ = true;

      return true;
   }

   struct invalidParam{ int param_;  invalidParam(int i) : param_(i) {} };
   struct invalidRow  { int param_;  invalidRow(int i) : param_(i) {} };

   void setParameter(int paramId, int rowId, double value)
       /// Use this function to populate the matrix with the parameters of the
       /// equations to be solved.  Use the setRHS() function with the same rowId
       /// as is used for the rowId above to set the right hand side of the equation
       /// whose parameters are set with this function.
       ///
       /// See comments in the class itself for more details
   {
      if(paramId >= n_ || paramId < 0)
         throw invalidParam(paramId);

      if(rowId >= n_ || rowId < 0)
         throw invalidRow(rowId);


      matrix_[rowId][paramId] = value;
   }

   void setRHS(int rowId, double value)
       ///  Use this function to set the right hand side value for the equation, identified by rowID.
       ///
       ///  You will have used setParamter(param: 0-(N-1), rowId, value: 0-(N-1)) to populate
       ///  the equation in the matrix.
   {
      if(rowId >= n_ || rowId < 0)
         throw invalidRow(rowId);

      rhs_[rowId] = value;
   }


   double result(int variableId)
       /// @return the computed variable value for the variable whose id is specified.  Note that
       /// the variable ids are in the same order as the parameter ids which are passed as parameter 1
       /// of setParameter().
   {
      if(variableId >= n_ || variableId < 0)
         throw invalidRow(variableId);

      return variables_[variableId];
   }

private:

   void evaluate_diag()
   {
      int i, j, k;

      double temp=0;

      for(i=0; i<n_; i++)
      {
          if(matrix_[i][i]==0)
          {
              for(j=0; j<n_; j++)
              {
                  if(j==i) continue;

                  if(matrix_[j][i] !=0 && matrix_[i][j]!=0)
                  {
                      for(k=0; k<n_; k++)
                      {
                          temp = matrix_[j][k];
                          matrix_[j][k] = matrix_[i][k];
                          matrix_[i][k] = temp;
                      }
                      temp = rhs_[j];
                      rhs_[j] = rhs_[i];
                      rhs_[i] = temp;

                      break;
                  }
              }
          }
      }
   }

   bool evaluate_rows()
      // return true on success.
   {
       int i, j, k;

       for(k=0; k<n_; k++){
           for(i=k+1; i<n_; i++){
               if(matrix_[k][k]==0){
                  return false;
               }
               double M = matrix_[i][k] / matrix_[k][k];
               for(j=k; j<n_; j++)
               {
                   matrix_[i][j] -= M * matrix_[k][j];
               }
               rhs_[i] -= M*rhs_[k];
           }
       }

       return true;
   }

   void evaluate_solution()
   {
       int  i, j;

       for(i=n_-1; i >= 0; i--)
       {
           double s = 0;

           for(j = i; j<n_; j++){
               s = s+matrix_[i][j]*variables_[j];
           }
           variables_[i] = (rhs_[i] - s) / matrix_[i][i];
       }
   }


}; // GuassianLpSolver

}  // namespace cxxtls

#endif
