#ifndef Hasher_h_included
#define Hasher_h_included

//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <string>

namespace cxxtls
{

///@file
///
///  This file defines a bunch of hash functors, all named Hasher<T>.  
///  The idea is to define good middle of the road algoritms -- not a complete
///  set optimized for all possible uses.

template<class T>
struct Hasher
  ///
  /// Hasher\<T\> is a class object which pretends to be a function which
  /// returns a semi-unique value for objects you pass it.  That is, it
  /// returns you an integer value which can be used as a hash code in
  /// various algoriths.  To use a Hasher, you  declare a Hasher object,
  /// then execute it like a function in  order to get a hash value.  For
  /// example:
  ///
  /// @code
  ///   Hasher<std::string> h;
  ///   int hashcode = h("stuff");
  /// @endcode
  ///
  /// Hash values are unsigned int.
  ///
  /// This basic implementation is meant for integral and pointer types.
  /// Other built in types are supported, feel free to follow the paradigm
  /// for your own class objects.  If you decide to create your own hasher,
  /// be careful not to fall into the trap of trying to make the hash value
  /// dependent on every single byte in your class object -- as is shown
  /// in the hasher's for string, double, and float.  The compiler will leave
  /// empty spaces in your class objects to align datatypes for maximum
  /// speed.  These empty spaces are not consistently valued -- and using
  /// in your hasher algorithm will result in inconsistent results.
  ///
  /// If your class object consists of pieces which can individually be
  /// hashed, you might consider a hash algorith that combines the hashes
  /// of all the separate pieces.  Say by orring them together after
  /// shifting, or something.
  ///
  /// Please ignore this line of the comment.  @{symbols/B}@.
  //
{
    unsigned int operator() (T const &t) { return (unsigned int)(t); }
};

template<>
struct Hasher<std::string>
  ///
  /// string specific hash algorithm.
  ///
{
    unsigned int operator() (std::string const &t)
    {
      unsigned int rv = t.size();

      std::string::const_iterator scan = t.begin(),
				  end  = t.end();
				
      while(scan != end)
      {
	rv <<= 1;
	
	rv |= *scan++;
      }

      return rv;
    }
};


template<>
struct Hasher<float>
/// Compute a hash value for a single precesion floating point number 
{
    unsigned int operator() (float const &t)
    {
      unsigned int rv = 0;

      char const *scan = (char const*)(&t);
      char const *end  = scan + sizeof(t);
				
      while(scan != end)
      {
	rv <<= 4;
	
	rv |= *scan++;
      }

      return rv;
    }
};

template<>
struct Hasher<double>
/// Compute a hash value for a double precesion floating point number 
{
    unsigned int operator() (double const &t)
    {
      unsigned int rv = 0;

      char const *scan = (char const*)(&t);
      char const *end  = scan + sizeof(t);
				
      while(scan != end)
      {
	rv <<= 4;
	
	rv |= *scan++;
      }

      return rv;
    }
};


} // namespace cxxtls

#endif
