#ifndef command_console_header_included_p
#define command_console_header_included_p
//
// Copyright 2002-2015, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure may be covered by the
// same copyright.  Some of them may be part of the GNU source distribution
// and you must obey the GPL copyright for any such files.


#include <vector>
#include <string>
#include <cxxtls/exception.h>

namespace cxxtls
{

struct UserControlC:  public Exception{};  // thrown when the user presses control C.


struct CommandConsole
    ///  The base class from which the Windows and the Curses versions derive.
{
     CommandConsole(std::string const &historyFile, std::vector<std::string> &historyLines)
     :  lineNumber_(0)
     ,  history_(historyLines)
     ,  historyFileName_(historyFile)
     {
        ///   Construct a command console given the name of a file, and a reference to the vector
        ///   of strings that are to hold the previous command history.
        ///
        ///   Normally, the ScriptLineStream constructs a CommandConsole and passes its own
        ///   history object as historyLines.

        readHistoryFile();   
     }

     static CommandConsole* getDefaultConsole(std::string const &historyFile, std::vector<std::string> &historyLines);
         ///  Open the default console for the current operating system, windows, linux, or 
         ///  whatever, and use the specified history file and container for the history lines.
         ///
         ///   Normally, the ScriptLineStream constructs a CommandConsole and passes its own
         ///   history object as historyLines.

    ~CommandConsole()
    {
        writeHistoryFile();
    }
     
     virtual bool readline(std::string &input, int &number, std::string const &dir) = 0; 
                                                                 ///< let the user enter a line of text and return it.
                                                                 ///
                                                                 ///  @param[in,out] d      The string to display and let the user edit.
                                                                 ///  @param[out]    number the line number of the returned string
                                                                 ///  @param[in]     dir    The directory to use for completions.
                                                                 ///
                                                                 ///  @return true if not end of file
                                                                 ///
                                                                 ///  @note  Pressing ^C kills all the children of the current
                                                                 ///         process but not the process itself.


     virtual bool kbhit() = 0; ///<  check for available keyboard data. sleep for 1/100 of second if none found.

  enum functionKeys 
  {
      ENDFILE= -1,         // FILE IO ERROR ON READ

#ifdef _MSC_VER
      F1=0x100 + 112,
      F2=0x100 + 113,
      F3=0x100 + 114,
      F4=0x100 + 115,
      F5=0x100 + 116,
      F6=0x100 + 117,
      F7=0x100 + 118,
      F8=0x100 + 119,
      F9=0x100 + 120,
     F10=0x100 + 121,
     F11=0x100 + 122,
     F12=0x100 + 123,

     INS=0x100 + 45,
     DEL=0x100 + 46,
     HOM=0x100 + 36,
     END=0x100 + 35,

     PUP=0x100 + 33,  // page up key
     PDN=0X100 + 34,  // page down key
      UP=0x100 + 38,
      DN=0X100 + 40,
      LT=0X100 + 37,
      RT=0x100 + 39,

      BS=0x08,        // back space key
      CTK=0x0b,       // control k
      CTQ=17,         // control Q
      CTW=23,         // control W

      SCREEN_RESIZED=0x100 + 1000
#else
      // linux/unix
#endif
  };


protected:

    int lineNumber_;

    std::vector<std::string> &history_;   // NOTE:  this is a reference!  The ScriptLineStream holds the actual container.

    std::string historyFileName_;

    void readHistoryFile();
    void writeHistoryFile();

};  // struct CommandConsole

} // namespace cxxtls
#endif
